/****************************************************************************
 *
 *	Copyright (c) 1999-2009, Fine Kernel Project, All rights reserved.
 *
 *	Redistribution and use in source and binary forms,
 *	with or without modification, are permitted provided that the
 *	following conditions are met:
 *
 *		- Redistributions of source code must retain the above
 *			copyright notice, this list of conditions and the
 *			following disclaimer.
 *
 *		- Redistributions in binary form must reproduce the above
 *			copyright notice, this list of conditions and the
 *			following disclaimer in the documentation and/or
 *			other materials provided with the distribution.
 *
 *		- Neither the name of the copyright holders nor the names
 *			of its contributors may be used to endorse or promote
 *			products derived from this software without specific
 *			prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *	"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *	LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *	FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *	COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *	STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 *	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *	POSSIBILITY OF SUCH DAMAGE. 
 *
 ****************************************************************************/
/****************************************************************************
 *
 *	Copyright (c) 1999-2009, Fine Kernel Project, All rights reserved.
 *
 *	本ソフトウェアおよびソースコードのライセンスは、基本的に
 *	「修正 BSD ライセンス」に従います。以下にその詳細を記します。
 *
 *	ソースコード形式かバイナリ形式か、変更するかしないかを問わず、
 *	以下の条件を満たす場合に限り、再頒布および使用が許可されます。
 *
 *	- ソースコードを再頒布する場合、上記の著作権表示、本条件一覧、
 *		および下記免責条項を含めること。
 *
 *	- バイナリ形式で再頒布する場合、頒布物に付属のドキュメント等の
 *		資料に、上記の著作権表示、本条件一覧、および下記免責条項を
 *		含めること。
 *
 *	- 書面による特別の許可なしに、本ソフトウェアから派生した製品の
 *		宣伝または販売促進に、本ソフトウェアの著作権者の名前または
 *		コントリビューターの名前を使用してはならない。
 *
 *	本ソフトウェアは、著作権者およびコントリビューターによって「現
 *	状のまま」提供されており、明示黙示を問わず、商業的な使用可能性、
 *	および特定の目的に対する適合性に関す暗黙の保証も含め、またそれ
 *	に限定されない、いかなる保証もないものとします。著作権者もコン
 *	トリビューターも、事由のいかんを問わず、損害発生の原因いかんを
 *	問わず、かつ責任の根拠が契約であるか厳格責任であるか(過失その
 *	他の)不法行為であるかを問わず、仮にそのような損害が発生する可
 *	能性を知らされていたとしても、本ソフトウェアの使用によって発生
 *	した(代替品または代用サービスの調達、使用の喪失、データの喪失、
 *	利益の喪失、業務の中断も含め、またそれに限定されない)直接損害、
 *	間接損害、偶発的な損害、特別損害、懲罰的損害、または結果損害に
 *	ついて、一切責任を負わないものとします。
 *
 ****************************************************************************/
#include "Ball_Main.h"
#include <FK/MatExample.h>

Ball_Main::Ball_Main(int argW, int argH)
	: fk_QtWidget(argW, argH)
{
	_view_mode = HIGH_MODE;
	return;
}

void Ball_Main::beforeLoop(void)
{
	fk_Material::initDefault();

	ground.setDivide(4);
	ground.setRadius(100.0);
	block.setSize(10.0, 10.0, 10.0);

	// ### VIEW POINT ###
	// 上の方から見た視点
	viewModel.glMoveTo(0.0, 400.0, 80.0);
	viewModel.glFocus(0.0, 30.0, 0.0);
	viewModel.glUpvec(0.0, 1.0, 0.0);

	// ### LIGHT ###
	light.setLightType(FK_POINT_LIGHT);
	light.setAttenuation(0.0, 0.0);
	lightModel.setShape(&light);
	lightModel.setMaterial(White);
	lightModel.glTranslate(-60.0, 60.0, 0.0);

	// ### GROUND ###
	groundModel.setShape(&ground);
	groundModel.setMaterial(LightGreen);
	groundModel.setSmoothMode(true);
	groundModel.loRotateWithVec(0.0, 0.0, 0.0, fk_X, -FK_PI/2.0);

	// ### VIEW BLOCK ###
	blockModel.setShape(&block);
	blockModel.setMaterial(Blue);
	blockModel.glMoveTo(60.0, 30.0, 0.0);
	blockModel.setParent(&groundModel);

	// ### BALL ###
	ball.getModel()->setMaterial(Red);
	ball.getModel()->setSmoothMode(true);
	
	// ### Model Entry ###
	scene.entryCamera(&viewModel);
	scene.entryModel(&lightModel);
	scene.entryModel(ball.getModel());
	scene.entryModel(&groundModel);
	scene.entryModel(&blockModel); 
	setScene(&scene);
	setInterval(1000);
}

bool Ball_Main::loop(void)
{
	if(getInterval() != 1) setInterval(1);

	// ボールを弾ませて, カメラの状態を取得。
	_view_mode = ball.draw(viewModel.getPosition());

	if(_view_mode == HIGH_MODE) {
		// カメラを上からの視点にする。
		viewModel.glMoveTo(0.0, 400.0, 80.0);
		viewModel.glFocus(0.0, 30.0, 0.0);
		viewModel.glUpvec(0.0, 1.0, 0.0);
		scene.entryModel(&blockModel);
	} else {
		// カメラをブロックからの視点にする。
		viewModel.glMoveTo(blockModel.getInhPosition());
		viewModel.glTranslate(0.0, 10.0, 0.0);
		viewModel.glFocus(ball.getPosition());
		viewModel.glUpvec(0.0, 1.0, 0.0);
		scene.removeModel(&blockModel);
	}

	// 地面をくるくる回転させましょう。
	groundModel.glRotateWithVec(0.0, 0.0, 0.0, fk_Y, 0.002);
	return true;
}
